/* ---------------------------------------------------------------------
   mx License Manager -- License Manager for commercial mx Extensions.

   Copyright (c) 2002, eGenix.com Software GmbH; mailto:info@egenix.com
   See the documentation for further copyright information or contact
   the author.
   ---------------------------------------------------------------------

   WARNING: Modifying the following code immediately causes a material
   breach of the eGenix.com Commercial License under which this code
   is made available to you, thereby making use of this software
   illegal under the terms of the license !

*/

/* --- Settings ----------------------------------------------------------- */

#ifndef MX_PRODUCT_ID
# define MX_PRODUCT_ID "mxODBC"
#endif
#ifndef MX_PRODUCT_MODULE
# define MX_PRODUCT_MODULE "mx.ODBC"
#endif
#ifndef MX_PRODUCT_VERSION
# define MX_PRODUCT_VERSION VERSION
#endif
#ifndef MX_LICENSE_MODULE
# define MX_LICENSE_MODULE "mx.ODBC.license"
#endif

/* --- Globals ------------------------------------------------------------ */

/* License notice */
static char mxLicenseManager_LicenseNotice[256] = \
	"NON-COMMERCIAL USE - 1 CPU License";

/* License Exception object */
static PyObject *mxLicenseManager_LicenseError = NULL;	

/* Time of initialization */
static double mxLicenseManager_Time = 0.0;

/* Flag telling us whether the manager was initialized or not. */
static int mxLicenseManager_Initialized = 0;


/* --- C API -------------------------------------------------------------- */

static
int mxLicenseManager_Init(PyObject *moddict)
{
    time_t ticks;

    /* Create error object */
    mxLicenseManager_LicenseError = PyErr_NewException(
				      MX_PRODUCT_MODULE".LicenseError", 
				      PyExc_ImportError,
				      NULL);
    if (mxLicenseManager_LicenseError == NULL)
	goto onError;
    if (moddict) {
	if (PyDict_SetItemString(moddict, 
				 "LicenseError", 
				 mxLicenseManager_LicenseError))
	    goto onError;
	/* Let the exception live in the module dict; we only keep a weak
	   reference */
	Py_DECREF(mxLicenseManager_LicenseError);
    }

    /* Get current time */
    if (time(&ticks) == (time_t) -1)
	goto onError;
    mxLicenseManager_Time = (double)ticks;
   
    /* We are now initialized */
    mxLicenseManager_Initialized = 1;
    return 0;

 onError:
    return -1;
}

/* Import the license module and load an attribute from it */

static
PyObject *mxLicenseManager_GetLicenseInfo(char *infoattr)
{
    PyObject *mod = 0, *v = 0;
    char *licmod = MX_LICENSE_MODULE;
    
    DPRINTF("Importing the license module %s ...\n", licmod);
    mod = PyImport_ImportModule(licmod);
    if (mod == NULL)
	goto onError;
    DPRINTF(" %s package found\n", licmod);
    v = PyObject_GetAttrString(mod, infoattr);
    if (v == NULL)
	goto onError;
    Py_DECREF(mod);
    DPRINTF(" license object %s found\n", infoattr);
    return v;
    
 onError:
    DPRINTF(" not found.\n");
    Py_XDECREF(mod);
    Py_XDECREF(v);
    return NULL;
}

#define mxR8BOX(value)			\
        (((value & 0xF00F0000) >> 16) |	\
	 ((value & 0x00F000F0) << 4) |	\
	 ((value & 0x0F00F000) >> 8) |	\
	 ((value & 0x00000F0F) << 20))

static
int mxLicenseManager_CheckLicense(PyObject *moddict)
{
    PyObject *v = NULL, *w = NULL;
    double packaging_time;
    double timeout = 7776142.513;
    unsigned int packaging_id;

    /* Init license manager */
    if (!mxLicenseManager_Initialized) {
	if (mxLicenseManager_Init(moddict))
	    goto onError;
    }

    /* Check license_key */
    v = mxLicenseManager_GetLicenseInfo("license_key");
    if (v == NULL)
	PyErr_Clear();
    else if (PyString_Check(v)) {

	if (strcmp(PyString_AS_STRING(v), 
		   "*** LICENSED FOR NON-COMMERCIAL USE ONLY ***") == 0) {
	    timeout = 31104913.413;
	}

	else {
	    /* XXX Check validity of licence_key */

	    /* Store license information in mxLicenseManager_LicenseNotice */
	    w = mxLicenseManager_GetLicenseInfo("license");
	    if (w == NULL)
		PyErr_Clear();
	    else if (PyString_Check(w)) {
		strncpy(mxLicenseManager_LicenseNotice, 
			PyString_AS_STRING(w),
			sizeof(mxLicenseManager_LicenseNotice) - 1);
	    }
	    else
		Py_Error(mxLicenseManager_LicenseError,
			 MX_LICENSE_MODULE".license must be a string");
	    Py_XDECREF(w);
	    timeout = 1e30;
	}
	DPRINTF("mxLicenseManager_CheckLicense: enabled license '%.200s'\n",
		mxLicenseManager_LicenseNotice);
    }
    else
	Py_Error(mxLicenseManager_LicenseError,
		 MX_LICENSE_MODULE".license_key must be a string");
    Py_XDECREF(v);
    
    /* Check packaging_id */
    v = mxLicenseManager_GetLicenseInfo("packaging_id");
    if (v == NULL)
	Py_Error(mxLicenseManager_LicenseError,
		 "mxLicenseManager can't find the "MX_PRODUCT_ID
		 " license information");
    packaging_id = PyInt_AsLong(v);
    Py_DECREF(v);
    if (PyErr_Occurred())
	Py_Error(mxLicenseManager_LicenseError,
		 "mxLicenseManager can't find usable license information for "
		 MX_PRODUCT_ID);
    packaging_time = mxR8BOX(packaging_id) * 4.0;
    if (mxLicenseManager_Time - packaging_time > timeout)
	Py_Error(mxLicenseManager_LicenseError,
		 "license for "MX_PRODUCT_ID" has expired");
    if (mxLicenseManager_Time < packaging_time)
	Py_Error(mxLicenseManager_LicenseError,
		 "license for "MX_PRODUCT_ID" is broken");

    return 0;

 onError:
    Py_XDECREF(v);
    Py_XDECREF(w);
    return -1;
}

/* === END OF LICENSE PROTECTION DEVICE ================================ */
